@@ -15,9 +15,6 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
- *
- * Authors: Debarshi Ray <debarshir@gnome.org>
- *          Willem van Engen <gnome@willem.engen.nl>
  */
 
 #include "config.h"
@@ -107,19 +104,19 @@ get_consumer_secret (GoaOAuthProvider *provider)
 static const gchar *
 get_request_uri (GoaOAuthProvider *provider)
 {
-  return "https://m.flickr.com/services/oauth/request_token";
+  return "http://m.flickr.com/services/oauth/request_token";
 }
 
 static const gchar *
 get_authorization_uri (GoaOAuthProvider *provider)
 {
-  return "https://m.flickr.com/services/oauth/authorize";
+  return "http://m.flickr.com/services/oauth/authorize";
 }
 
 static const gchar *
 get_token_uri (GoaOAuthProvider *provider)
 {
-  return "https://m.flickr.com/services/oauth/access_token";
+  return "http://m.flickr.com/services/oauth/access_token";
 }
 
 static const gchar *
@@ -168,7 +165,7 @@ get_identity_sync (GoaOAuthProvider  *provider,
                                       goa_oauth_provider_get_consumer_secret (provider),
                                       access_token,
                                       access_token_secret,
-                                      "https://api.flickr.com/services/rest",
+                                      "http://api.flickr.com/services/rest",
                                       FALSE);
   call = rest_proxy_new_call (proxy);
   rest_proxy_call_add_param (call, "method", "flickr.test.login");
