@@ -14,9 +14,6 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
- *
- * Authors: David Zeuthen <davidz@redhat.com>
- *          Debarshi Ray <debarshir@gnome.org>
  */
 
 #include "config.h"
@@ -145,16 +142,13 @@ get_scope (GoaOAuth2Provider *provider)
 	 "https://www.googleapis.com/auth/cloudprint "
 
          /* Google Talk */
-         "https://www.googleapis.com/auth/googletalk "
-
-         /* Google Tasks - undocumented */
-         "https://www.googleapis.com/auth/tasks";
+         "https://www.googleapis.com/auth/googletalk";
 }
 
 static guint
 get_credentials_generation (GoaProvider *provider)
 {
-  return 8;
+  return 7;
 }
 
 static const gchar *
