@@ -14,8 +14,6 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
- *
- * Authors: Ray Strode
  */
 
 #ifndef __GOA_ALARM_H__
@@ -53,7 +51,10 @@ struct _GoaAlarmClass
 
 GType goa_alarm_get_type (void);
 
-GoaAlarm *goa_alarm_new (GDateTime *time);
+GoaAlarm *goa_alarm_new (void);
+void goa_alarm_set_time (GoaAlarm     *alarm,
+                         GDateTime    *time,
+                         GCancellable *cancellable);
 GDateTime *goa_alarm_get_time (GoaAlarm *alarm);
 G_END_DECLS
 #endif /* __GOA_ALARM_H__ */
