@@ -14,8 +14,6 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see <http://www.gnu.org/licenses/>.
- *
- * Author: Ray Strode
  */
 
 #include "config.h"
@@ -53,8 +51,13 @@ struct _GoaKerberosIdentityPrivate
   guint          expiration_time_idle_id;
 
   GoaAlarm     *expiration_alarm;
+  GCancellable *expiration_alarm_cancellable;
+
   GoaAlarm     *expiring_alarm;
+  GCancellable *expiring_alarm_cancellable;
+
   GoaAlarm     *renewal_alarm;
+  GCancellable *renewal_alarm_cancellable;
 
   VerificationLevel cached_verification_level;
   guint             is_signed_in_idle_id;
@@ -107,6 +110,8 @@ goa_kerberos_identity_dispose (GObject *object)
   GoaKerberosIdentity *self = GOA_KERBEROS_IDENTITY (object);
 
   G_LOCK (identity_lock);
+  clear_alarms (self);
+
   g_clear_object (&self->priv->renewal_alarm);
   g_clear_object (&self->priv->expiring_alarm);
   g_clear_object (&self->priv->expiration_alarm);
@@ -233,7 +238,7 @@ get_identifier (GoaKerberosIdentity  *self,
   krb5_principal principal;
   krb5_error_code error_code;
   char *unparsed_name;
-  char *identifier = NULL;
+  char *identifier;
 
   if (self->priv->credentials_cache == NULL)
     return NULL;
@@ -279,14 +284,12 @@ get_identifier (GoaKerberosIdentity  *self,
       g_debug ("GoaKerberosIdentity: Error parsing principal identity name: %s",
                error_message);
       krb5_free_error_message (self->priv->kerberos_context, error_message);
-      goto out;
+      return NULL;
     }
 
   identifier = g_strdup (unparsed_name);
   krb5_free_unparsed_name (self->priv->kerberos_context, unparsed_name);
 
-out:
-  krb5_free_principal (self->priv->kerberos_context, principal);
   return identifier;
 }
 
@@ -296,6 +299,9 @@ goa_kerberos_identity_init (GoaKerberosIdentity *self)
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                             GOA_TYPE_KERBEROS_IDENTITY,
                                             GoaKerberosIdentityPrivate);
+  self->priv->expiration_alarm = goa_alarm_new ();
+  self->priv->expiring_alarm = goa_alarm_new ();
+  self->priv->renewal_alarm = goa_alarm_new ();
 }
 
 static void
@@ -728,6 +734,8 @@ on_renewal_alarm_fired (GoaAlarm            *alarm,
   g_return_if_fail (GOA_IS_ALARM (alarm));
   g_return_if_fail (GOA_IS_KERBEROS_IDENTITY (self));
 
+  g_clear_object (&self->priv->renewal_alarm_cancellable);
+
   if (self->priv->cached_verification_level == VERIFICATION_LEVEL_SIGNED_IN)
     {
       g_debug ("GoaKerberosIdentity: renewal alarm fired for signed-in identity");
@@ -752,6 +760,8 @@ on_expiring_alarm_fired (GoaAlarm            *alarm,
   g_return_if_fail (GOA_IS_ALARM (alarm));
   g_return_if_fail (GOA_IS_KERBEROS_IDENTITY (self));
 
+  g_clear_object (&self->priv->expiring_alarm_cancellable);
+
   if (self->priv->cached_verification_level == VERIFICATION_LEVEL_SIGNED_IN)
     {
       g_debug ("GoaKerberosIdentity: expiring alarm fired for signed-in identity");
@@ -759,38 +769,25 @@ on_expiring_alarm_fired (GoaAlarm            *alarm,
     }
 }
 
-static gboolean
-unref_alarm (GoaAlarm *alarm)
-{
-  g_object_unref (G_OBJECT (alarm));
-  return G_SOURCE_REMOVE;
-}
-
 static void
-clear_alarm_and_unref_on_idle (GoaKerberosIdentity  *self,
-                     GoaAlarm            **alarm)
+set_alarm (GoaKerberosIdentity  *self,
+           GoaAlarm             *alarm,
+           GDateTime            *alarm_time,
+           GCancellable        **cancellable)
 {
-  if (!*alarm)
-    return;
-
-  g_idle_add ((GSourceFunc) unref_alarm, *alarm);
-  *alarm = NULL;
-}
-
-static void
-reset_alarm (GoaKerberosIdentity  *self,
-             GoaAlarm            **alarm,
-             GDateTime            *alarm_time)
-{
-  GDateTime *old_alarm_time = NULL;
+  GDateTime *old_alarm_time;
 
   G_LOCK (identity_lock);
-  if (*alarm)
-    old_alarm_time = goa_alarm_get_time (*alarm);
+  old_alarm_time = goa_alarm_get_time (alarm);
   if (old_alarm_time == NULL || !g_date_time_equal (alarm_time, old_alarm_time))
     {
-      clear_alarm_and_unref_on_idle (self, alarm);
-      *alarm = goa_alarm_new (alarm_time);
+      GCancellable *new_cancellable;
+
+      new_cancellable = g_cancellable_new ();
+      goa_alarm_set_time (alarm, alarm_time, new_cancellable);
+
+      g_clear_object (cancellable);
+      *cancellable = new_cancellable;
     }
   G_UNLOCK (identity_lock);
 
@@ -799,35 +796,24 @@ reset_alarm (GoaKerberosIdentity  *self,
 static void
 disconnect_alarm_signals (GoaKerberosIdentity *self)
 {
-  if (self->priv->renewal_alarm)
-    {
-      g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->renewal_alarm),
-                                            G_CALLBACK (on_renewal_alarm_fired),
-                                            self);
-      g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->renewal_alarm),
-                                            G_CALLBACK (on_renewal_alarm_rearmed),
-                                            self);
-    }
-
-  if (self->priv->expiring_alarm)
-    {
-      g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiring_alarm),
-                                            G_CALLBACK (on_expiring_alarm_fired),
-                                            self);
-      g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiring_alarm),
-                                            G_CALLBACK (on_expiring_alarm_rearmed),
-                                            self);
-    }
-
-  if (self->priv->expiration_alarm)
-    {
-      g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiration_alarm),
-                                            G_CALLBACK (on_expiration_alarm_rearmed),
-                                            self);
-      g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiration_alarm),
-                                            G_CALLBACK (on_expiration_alarm_fired),
-                                            self);
-    }
+  g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->renewal_alarm),
+                                        G_CALLBACK (on_renewal_alarm_fired),
+                                        self);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->renewal_alarm),
+                                        G_CALLBACK (on_renewal_alarm_rearmed),
+                                        self);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiring_alarm),
+                                        G_CALLBACK (on_expiring_alarm_fired),
+                                        self);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiration_alarm),
+                                        G_CALLBACK (on_expiration_alarm_rearmed),
+                                        self);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiration_alarm),
+                                        G_CALLBACK (on_expiration_alarm_fired),
+                                        self);
+  g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->expiring_alarm),
+                                        G_CALLBACK (on_expiring_alarm_rearmed),
+                                        self);
 }
 
 static void
@@ -884,9 +870,15 @@ reset_alarms (GoaKerberosIdentity *self)
 
   disconnect_alarm_signals (self);
 
-  reset_alarm (self, &self->priv->renewal_alarm, renewal_time);
-  reset_alarm (self, &self->priv->expiring_alarm, expiring_time);
-  reset_alarm (self, &self->priv->expiration_alarm, expiration_time);
+  set_alarm (self,
+             self->priv->renewal_alarm,
+             renewal_time, &self->priv->renewal_alarm_cancellable);
+  set_alarm (self,
+             self->priv->expiring_alarm,
+             expiring_time, &self->priv->expiring_alarm_cancellable);
+  set_alarm (self,
+             self->priv->expiration_alarm,
+             expiration_time, &self->priv->expiration_alarm_cancellable);
 
   g_date_time_unref (renewal_time);
   g_date_time_unref (expiring_time);
@@ -894,13 +886,24 @@ reset_alarms (GoaKerberosIdentity *self)
   connect_alarm_signals (self);
 }
 
+static void
+cancel_and_clear_cancellable (GCancellable **cancellable)
+{
+  if (cancellable == NULL)
+    return;
+
+  if (!g_cancellable_is_cancelled (*cancellable))
+    g_cancellable_cancel (*cancellable);
+
+  g_clear_object (cancellable);
+}
+
 static void
 clear_alarms (GoaKerberosIdentity *self)
 {
-  disconnect_alarm_signals (self);
-  clear_alarm_and_unref_on_idle (self, &self->priv->renewal_alarm);
-  clear_alarm_and_unref_on_idle (self, &self->priv->expiring_alarm);
-  clear_alarm_and_unref_on_idle (self, &self->priv->expiration_alarm);
+  cancel_and_clear_cancellable (&self->priv->renewal_alarm_cancellable);
+  cancel_and_clear_cancellable (&self->priv->expiring_alarm_cancellable);
+  cancel_and_clear_cancellable (&self->priv->expiration_alarm_cancellable);
 }
 
 static gboolean
